<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs" version="2.0">
  
  <xsl:variable name="svnId">$Id: Makefile.xsl 326 2014-04-13 21:00:33Z zw $</xsl:variable>
  
  <xsl:strip-space elements="*"/>
  <xsl:output method="text" encoding="UTF-8" indent="no"/>
  
  <!-- Root template -->
  <xsl:template match="/">
    <xsl:value-of select="concat('## Generated by ', $svnId, '&#10;')"/>
    <xsl:apply-templates/>
  </xsl:template>
  
  <!-- Book element -->
  <xsl:template match="book">
    <xsl:value-of select="concat('## from ', @svnId, '&#10;')"/>
    <xsl:value-of select="concat('imgpath=', @imgpath, '&#10;')"/>
    <xsl:value-of select="concat('jpgdir=', @jpg, '&#10;')"/>
    <xsl:value-of select="concat('pdfdir=', @pdf, '&#10;')"/>
    <xsl:variable name="sorted-dirs" as="xs:string*">
      <xsl:for-each select="//img/@src">
        <xsl:sort select="."/>
        <xsl:sequence select="substring-before(., '/')"/>
      </xsl:for-each>
    </xsl:variable>
    <xsl:variable name="dirs" as="xs:string*" select="distinct-values($sorted-dirs)"/>
    <xsl:text>dirs=</xsl:text>
    <xsl:value-of select="$dirs" separator=" "/>
    <xsl:text>&#10;</xsl:text>
    <xsl:variable name="files" as="xs:string*" select="//img/@src"/>
    <xsl:text>files=</xsl:text>
    <xsl:value-of select="$files" separator=" "/>
    <xsl:text>&#10;</xsl:text>
    <xsl:variable name="images" as="xs:string*" select="/book/images/img/@src"/>
    <xsl:text>images=</xsl:text>
    <xsl:value-of select="$images" separator=" "/>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>jpgs=${addprefix ${imgpath}/${jpgdir}/, ${addsuffix .jpg, ${files}}}&#10;</xsl:text>
    <xsl:text>pdfs=${addprefix ${imgpath}/${pdfdir}/, ${addsuffix .pdf, ${files}}}&#10;</xsl:text>
    <xsl:text>imgs=${addprefix ${imgpath}/${pdfdir}/, ${addsuffix .pdf, ${images}}}&#10;</xsl:text>
    <xsl:text>incl=backmatter.tex frontmatter.tex zwsizes.sty zwstyl.sty&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>.PHONY: all dirs delimgs&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>all: BharatKeRang.pdf Predsadky.pdf Obalka.pdf&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>Predsadky.pdf Obalka.pdf: zwstyl.sty zwpath.sty ${imgs}&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>BharatKeRang.pdf: ${incl} ${jpgs} ${pdfs}&#10;&#10;</xsl:text>
    <xsl:text>%.pdf: %.tex&#10;</xsl:text>
    <xsl:text>&#9;zwxelatex $&lt;&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>delimgs:&#10;&#9;./delimgs.pl ${imgpath} ${jpgdir} ${pdfdir} ${files}&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>dirs: delimgs&#10;&#9;./mkdirs.pl ${imgpath} ${jpgdir} ${pdfdir} ${dirs}&#10;</xsl:text>
    <xsl:text>&#9;./auxcopy.sh&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>${imgpath}/${jpgdir}/%.jpg: ${imgpath}/%.tif&#10;</xsl:text>
    <xsl:text>&#9;convert -sample 700x1000 $&lt; $@&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>${imgpath}/${pdfdir}/%.pdf: ${imgpath}/%.tif&#10;</xsl:text>
    <xsl:text>&#9;./image2pdf.sh $&lt; $@&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>
  
</xsl:stylesheet>
