<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:zw="http://icebearsoft.euweb.cz/ns/private"
  xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs zw" version="2.0">
  
  <xsl:variable name="svnId">$Id: xml2tex.xsl 326 2014-04-13 21:00:33Z zw $</xsl:variable>
  
  <xsl:strip-space elements="*"/>
  <xsl:output method="text" encoding="UTF-8" indent="no"/>
  
  <!-- Layout constants -->
  <xsl:variable name="pagewidth" as="xs:decimal" select="210"/>
  <xsl:variable name="pageheight" as="xs:decimal" select="297"/>
  <xsl:variable name="cropgap" as="xs:decimal" select="5"/>
  
  <!-- Layout dimensions, \unitlength=1mm -->
  <xsl:variable name="baselineskip" as="xs:decimal" select="xs:decimal(//sizes/@normalsize)"/>
  
  <xsl:variable name="linewidth" as="xs:decimal"
    select="$pagewidth - xs:decimal(//layout/@leftmargin) - xs:decimal(//layout/@rightmargin)"/>
  
  <xsl:variable name="cropwidth" as="xs:decimal" select="$pagewidth + $cropgap"/>
  
  <xsl:variable name="oddspine" as="xs:decimal"
   select="-xs:decimal(//layout/@leftmargin)"/>
  
  <xsl:variable name="evenspine" as="xs:decimal"
   select="$linewidth + xs:decimal(//layout/@leftmargin)"/>
  
  <xsl:variable name="oddouter" as="xs:decimal" select="$cropwidth + $oddspine"/>
  
  <xsl:variable name="evenouter" as="xs:decimal" select="- xs:decimal(//layout/@rightmargin) - $cropgap"/>
  
  <xsl:variable name="footskip" as="xs:decimal" select="xs:decimal(//layout/@footskip)"/>
  
  <xsl:variable name="topskip" as="xs:decimal" select="xs:decimal(//layout/@topskip)"/>
  
  <xsl:variable name="topmargin" as="xs:decimal" select="xs:decimal(//layout/@topmargin)"/>
  
  <xsl:variable name="botmargin" as="xs:decimal"
   select="if (//layout/@botmargin eq 'default') then $topmargin else
   xs:decimal(//layout/@botmargin)"/>
  
  <xsl:variable name="numlines" as="xs:decimal"
   select="floor(($pageheight - $topmargin - $topskip - $botmargin - $footskip) div $baselineskip)"/>
  
  <xsl:variable name="top" as="xs:decimal" select="$numlines * $baselineskip"/>
  
  <xsl:variable name="XTop" as="xs:decimal" select="$top + $topskip"/>
  
  <xsl:variable name="Top" as="xs:decimal" select="$XTop - 3"/>
  
  <xsl:variable name="HTdiff" as="xs:decimal"
    select="$pageheight - $topmargin - $XTop - $footskip - $botmargin"/>
  
  <xsl:variable name="adjhead" as="xs:decimal"
    select="if (//layout/@adjust eq 'headsep') then $HTdiff else 0"/>
  
  <xsl:variable name="adjfoot" as="xs:decimal"
    select="if (//layout/@adjust eq 'headsep') then 0 else $HTdiff"/>
  
  <xsl:variable name="BOTTOM" as="xs:decimal"
   select="- $adjfoot - $footskip - $botmargin - $cropgap"/>
  
  <xsl:variable name="TOP" as="xs:decimal"
   select="$XTop + $adjhead + $topmargin + $cropgap"/>
  
  <xsl:variable name="cropheight" as="xs:decimal" select="$pageheight + 2 * $cropgap"/>
  
  <!-- Define the dimensions for LaTeX -->
  <xsl:template name="dimensions">
    <xsl:value-of select="zw:defTeXdim('ZWlinewidth', $linewidth)"/>
    <xsl:value-of select="zw:defTeXdim('ZWcropwidth', $cropwidth)"/>
    <xsl:value-of select="zw:defTeXdim('ZWcropheight', $cropheight)"/>
    <xsl:value-of select="zw:defTeXdim('ZWTop', $Top)"/>
    <xsl:value-of select="zw:defTeXdim('horizlines', 1 + floor($XTop div 15))"/>
    <xsl:value-of select="zw:defTeXdim('vertlines', 1 + floor($linewidth div 15))"/>
    <xsl:value-of select="zw:defTeXdim('Horizlines', 1 + floor($XTop div 75))"/>
    <xsl:value-of select="zw:defTeXdim('Vertlines', 1 + floor($linewidth div 75))"/>
  </xsl:template>
  
  <!-- Root template -->
  <xsl:template match="/">
    <xsl:value-of select="concat('%% Generated by ', $svnId, '&#10;')"/>
    <xsl:apply-templates/>
  </xsl:template>
  
  <!-- book element -->
  <xsl:template match="book">
    <xsl:value-of select="concat('%% from ', @svnId, '&#10;')"/>
    <xsl:text>\errorcontextlines 999&#10;\nonstopmode&#10;</xsl:text>
    <xsl:text>\documentclass[12pt]{book}&#10;</xsl:text>
    <xsl:text>\usepackage{ifxetex}\RequireXeTeX&#10;</xsl:text>
    <xsl:variable name="imgpath">
      <xsl:value-of select="concat('\def\imgpath{', @imgpath, '/')"/>
      <xsl:choose>
        <xsl:when test="@mode eq 'pdf'">
          <xsl:value-of select="concat(@pdf, '}&#10;')"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat(@jpg, '}&#10;')"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:value-of select="$imgpath"/>
    <xsl:result-document href="zwpath.sty" method="text" encoding="utf-8" indent="no">
      <xsl:text>\ProvidesPackage{zwpath}&#10;</xsl:text>
      <xsl:value-of select="$imgpath"/>
      <xsl:value-of select="concat('\def\Ipath{', @imgpath, '/', @pdf, '}&#10;')"/>
      <xsl:text>\def\img[#1]#2{\includegraphics[#1]{\Ipath/#2}}&#10;</xsl:text>
    </xsl:result-document>
    <xsl:apply-templates/>
    <xsl:text>\end{document}&#10;</xsl:text>
  </xsl:template>
  
  <!-- Definition of \baselineskip for font sizes -->
  <xsl:template match="sizes">
    <xsl:apply-templates select="@*"/>
    <xsl:text>\usepackage{zwsizes}&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="sizes/@*">
    <xsl:value-of select="concat('\def\ZW', name(), '{', ., 'mm}&#10;')"/>
  </xsl:template>
  
  <!-- Element with options for zwpagelayout -->
  <xsl:template match="layout">
    <xsl:text>\usepackage[a4,color,</xsl:text>
    <xsl:if test="@cropmarks eq 'no'">
      <xsl:text>no</xsl:text>
    </xsl:if>
    <xsl:text>cropmarks</xsl:text>
    <xsl:apply-templates select="/book/@title,@* except @cropmarks"/>
    <xsl:text>]{zwpagelayout}[2012/05/20]&#10;</xsl:text>
    <xsl:call-template name="dimensions"/>
    <xsl:text>\usepackage{zwstyl}&#10;</xsl:text>
    <xsl:if test="/book/@grid eq 'yes'">
      <xsl:text>\gridtrue&#10;</xsl:text>
    </xsl:if>
    <xsl:text>\begin{document}&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="book/@title | layout/@author">
    <xsl:variable name="name" as="xs:string"
      select="if (name() = 'title') then 'croptitle' else name()"/>
    <xsl:value-of select="concat(',', $name, '={', ., '}')"/>
  </xsl:template>
  
  <xsl:template match="layout/@adjust">
    <xsl:value-of select="concat(',adjust', .)"/>
  </xsl:template>
  
  <xsl:template match="layout/@botmargin">
    <xsl:text>,botmargin</xsl:text>
    <xsl:if test=". ne 'default'">
      <xsl:value-of select="concat('=', ., 'mm')"/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="@*">
    <xsl:value-of select="concat(',', name(), '=', ., 'mm')"/>
  </xsl:template>
  
  <!-- Inclusion of a frontmatter and a backatter (defined directly in TeX -->
  <xsl:template match="frontmatter|backmatter">
    <xsl:value-of select="concat('\input{', name(), '}&#10;')"/>
  </xsl:template>
  
  <!-- part defines a chapter/section -->
  <xsl:template match="part">
    <xsl:text>\begin{ZWpart}&#10;</xsl:text>
    <xsl:apply-templates select="@grid"/>
    <!-- Title for toc -->
    <xsl:value-of select="concat('\def\toctype{', @type, '}&#10;')"/>
    <xsl:apply-templates select="title" mode="toc"/>
    <!-- Text boxes -->
    <xsl:if test="page/text">
      <xsl:variable name="vbox" as="element()" select="zw:vbox(page/text)"/>
      <xsl:text>\begin{sectionbox}{</xsl:text>
      <xsl:for-each select="$vbox/parshape">
        <xsl:value-of select="concat(@number, ',', @indent, ',', @width, ';')"/>
      </xsl:for-each>
      <xsl:text>}&#10;</xsl:text>
      <xsl:choose>
        <xsl:when test="@draft eq 'yes' or /book/@mode eq 'draft'">
          <xsl:text>\ZWrules&#10;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="para|vskip"/>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:text>\end{sectionbox}&#10;</xsl:text>
    </xsl:if>
    <xsl:apply-templates select="page"/>
    <xsl:text>\end{ZWpart}&#10;</xsl:text>
  </xsl:template>
  
  <!-- Title contents -->
  <xsl:template match="line" mode="toc">
    <xsl:choose>
      <xsl:when test="@lang eq 'cs'">
        <xsl:value-of select="concat('\def\toctext{', zw:toc(.))"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of
          select="concat(' / \text', zw:language(@lang), '{', zw:toc(.), '}')"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="position() eq last()">
      <xsl:text>}&#10;</xsl:text>
    </xsl:if>
  </xsl:template>
  
  <xsl:function name="zw:toc">
    <xsl:param name="elem"/>
    <xsl:value-of select="if ($elem/toc) then $elem/toc else $elem/titletext"/>
  </xsl:function>
  
  <!-- Text elements -->
  <xsl:template match="vskip">
    <xsl:value-of
      select="concat('\ZWvskip{', if (@vb) then @vb else 1, '}&#10;')"/>
  </xsl:template>
  
  <xsl:template match="para">
    <xsl:choose>
      <xsl:when test="@indent eq 'yes'">
        <xsl:text>\indent</xsl:text>
      </xsl:when>
      <xsl:when test="@indent eq 'no'">
        <xsl:text>\noindent</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\leavevmode</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>\ZWstrut&#10;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>\par&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="span">
    <xsl:value-of select="concat('\text', zw:language(@lang), '{', ., '}')"/>
  </xsl:template>
  
  <xsl:template match="denser|looser">
    <xsl:value-of select="concat('\', name())"/>
    <xsl:if test="@lines">
      <xsl:value-of select="concat('[', @lines, ']')"/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="emergencystretch">
    <xsl:value-of select="concat('\emergencystretch=', @value, ' ')"/>
  </xsl:template>
  
  <!-- Page description -->
  <xsl:template match="page">
    <xsl:variable name="pagepos" as="xs:integer">
      <xsl:choose>
        <xsl:when test="@newpage eq 'even'">
          <xsl:sequence select="0"/>
        </xsl:when>
        <xsl:when test="@newpage eq 'odd'">
          <xsl:sequence select="1"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="pages" as="xs:integer*">
            <xsl:for-each-group select="(preceding::page)"
              group-starting-with="page[@newpage ne '']">
              <xsl:variable name="startpos" as="xs:integer"
               select="if (current-group()[1]/@newpage eq 'even') then 0 else 1"/>
              <xsl:value-of select="(count(current-group()) + $startpos) mod 2"/>
            </xsl:for-each-group>
          </xsl:variable>
          <xsl:value-of select="if (not(empty($pages))) then $pages[last()]
            else (1 + count((preceding::page))) mod 2"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="@newpage eq 'odd'">
        <xsl:text>\NewOddPage&#10;</xsl:text>
      </xsl:when>
      <xsl:when test="@newpage eq 'even'">
        <xsl:text>\NewEvenPage&#10;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\clearpage&#10;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="@thispagestyle">
      <xsl:value-of select="concat('\thispagestyle{', @thispagestyle, '}&#10;')"/>
    </xsl:if>
    <xsl:text>\begin{page}{</xsl:text>
    <xsl:apply-templates select="@grid"/>
    <xsl:text>}&#10;</xsl:text>
    <!-- Contents -->
    <xsl:apply-templates>
      <xsl:with-param name="pagepos" as="xs:integer" select="$pagepos" tunnel="yes"/>
    </xsl:apply-templates>
    <xsl:text>\end{page}&#10;</xsl:text>
  </xsl:template>
  
  <!-- Titlebox -->
  <xsl:template match="titlebox">
    <xsl:text>\setbox\ZWbox=\vtop{\hsize=</xsl:text>
    <xsl:value-of select="zw:TeXdim(width)"/>
    <xsl:text>\hrule height 0mm\par&#10;</xsl:text>
    <xsl:choose>
      <xsl:when test="@textalign eq 'left'">
        <xsl:text>\raggedright </xsl:text>
      </xsl:when>
      <xsl:when test="@textalign eq 'right'">
        <xsl:text>\raggedleft </xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\centering </xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:choose>
      <xsl:when test="../@type eq 'section'">
        <xsl:text>\Large&#10;</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>\huge&#10;</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates select="../../title"/>
    <xsl:text>}&#10;</xsl:text>
    <xsl:call-template name="put-xy">
      <xsl:with-param name="x-shift">
        <xsl:value-of select="if (@halign eq 'right') then zw:milimeters(width) else 0"/>
      </xsl:with-param>
    </xsl:call-template>
    <xsl:text>\box\ZWbox}&#10;</xsl:text>
    <xsl:text>\addcontentsline{toc}{\toctype}{\toctext}&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template match="line">
    <xsl:if test="@lang ne 'cs'">
      <xsl:value-of select="concat('\text', zw:language(@lang), '{')"/>
    </xsl:if>
    <xsl:apply-templates select="titletext"/>
    <xsl:if test="@lang ne 'cs'">
      <xsl:text>}</xsl:text>
    </xsl:if>
    <xsl:text>\par&#10;</xsl:text>
  </xsl:template>
  
  <!-- text box -->
  <xsl:template match="text">
    <xsl:variable name="vbox" as="element()" select="zw:vbox(.)"/>
    <xsl:text>\ZW</xsl:text>
    <xsl:choose>
      <xsl:when test="@valign eq 'top'">
        <xsl:text>vtop</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>vbox</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:value-of select="concat('{', $vbox/@height+.01, '}{', $vbox/@width, '}&#10;')"/>
    <xsl:call-template name="put-xy">
      <xsl:with-param name="x-shift">
        <xsl:value-of select="if (@halign eq 'right') then $vbox/@width else 0"/>
      </xsl:with-param>
    </xsl:call-template>
    <xsl:text>\box\ZWbox}&#10;</xsl:text>
  </xsl:template>
  
  <!-- Image, may have a caption -->
  <xsl:template match="img">
    <xsl:call-template name="put-xy"/>
    <xsl:text>\makebox(0,0)</xsl:text>
    <xsl:if test="@pos">
      <xsl:value-of select="concat('[', @pos, ']')"/>
    </xsl:if>
    <xsl:text>{</xsl:text>
    <xsl:choose>
      <xsl:when test="caption">
        <xsl:text>\setbox\ZWbox=\hbox{</xsl:text>
        <xsl:call-template name="includegraphics"/>
        <xsl:text>}%&#10;</xsl:text>
        <xsl:text>\vbox{\hsize\wd\ZWbox \centering \small \box\ZWbox \vb[.4]&#10;</xsl:text>
        <xsl:apply-templates select="caption"/>
        <xsl:text>}</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="includegraphics"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>}}&#10;</xsl:text>
  </xsl:template>
  
  <xsl:template name="includegraphics">
    <xsl:text>\includegraphics[</xsl:text>
    <xsl:if test="height">
      <xsl:value-of select="concat('height=', zw:TeXdim(height))"/>
    </xsl:if>
    <xsl:if test="width">
      <xsl:value-of select="concat('width=', zw:TeXdim(width))"/>
    </xsl:if>
    <xsl:value-of select="concat(']{\imgpath/', @src, '}')"/>
  </xsl:template>
  
  <!-- Prepare \put optionally x-shifted -->
  <xsl:template name="put-xy">
    <xsl:param name="x-shift" as="xs:decimal" select="0"/>
    <xsl:param name="pagepos" as="xs:integer" tunnel="yes"/>
    <xsl:variable name="x" as="xs:decimal">
      <xsl:variable name="xpos" as="xs:decimal">
        <xsl:choose>
          <xsl:when test="x eq 'left'">
            <xsl:sequence select="0"/>
          </xsl:when>
          <xsl:when test="x eq 'right'">
            <xsl:sequence select="$linewidth"/>
          </xsl:when>
          <xsl:when test="x eq 'center'">
            <xsl:sequence select="0.5 * $linewidth"/>
          </xsl:when>
          <xsl:when test="x eq 'spine'">
            <xsl:sequence select="if ($pagepos &gt; 0) then $oddspine else $evenspine"/>
          </xsl:when>
          <xsl:when test="x eq 'outer'">
            <xsl:sequence select="if ($pagepos &gt; 0) then $oddouter else $evenouter"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:sequence select="xs:decimal(x)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <xsl:sequence select="$xpos - $x-shift"/>
    </xsl:variable>
    <xsl:variable name="y" as="xs:decimal">
      <xsl:choose>
        <xsl:when test="y eq 'bottom'">
          <xsl:sequence select="0"/>
        </xsl:when>
        <xsl:when test="y eq 'top'">
          <xsl:sequence select="$top"/>
        </xsl:when>
        <xsl:when test="y eq 'Top'">
          <xsl:sequence select="$Top"/>
        </xsl:when>
        <xsl:when test="y eq 'TOP'">
          <xsl:sequence select="$TOP"/>
        </xsl:when>
        <xsl:when test="y eq 'BOTTOM'">
          <xsl:sequence select="$BOTTOM"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:variable name="Y" as="xs:decimal" select="xs:decimal(y)"/>
          <xsl:sequence
            select="if (name() eq 'text') then round($Y div $baselineskip) * $baselineskip else $Y"
          />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:value-of select="concat('\put(', $x, ',', $y, '){')"/>
  </xsl:template>
  
  <!-- Override grid setting -->
  <xsl:template match="@grid">
    <xsl:choose>
      <xsl:when test=". eq 'yes'">
        <xsl:text>\gridtrue&#10;</xsl:text>
      </xsl:when>
      <xsl:when test=". eq 'no'">
        <xsl:text>\gridfalee&#10;</xsl:text>
      </xsl:when>
    </xsl:choose>
  </xsl:template>
  
  <!-- Expand the language name -->
  <xsl:function name="zw:language" as="xs:string">
    <xsl:param name="lang"/>
    <xsl:choose>
      <xsl:when test="$lang eq 'cs'">
        <xsl:text>czech</xsl:text>
      </xsl:when>
      <xsl:when test="$lang eq 'hi'">
        <xsl:text>hindi</xsl:text>
      </xsl:when>
      <xsl:when test="$lang eq 'ur'">
        <xsl:text>urdu</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!-- convert the value od "dimension" element to milimeters, if @value missing then 1 -->
  <xsl:function name="zw:milimeters" as="xs:decimal">
    <xsl:param name="elem" as="element()"/>
    <xsl:variable name="value" as="xs:decimal"
     select="if ($elem/@value) then xs:decimal($elem/@value) else 1"/>
    <xsl:choose>
      <xsl:when test="$elem/@unit eq 'mm'">
        <xsl:value-of select="$value"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'cm'">
        <xsl:value-of select="10 * $value"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'in'">
        <xsl:value-of select="25.4 * $value"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'pt'">
        <xsl:value-of select="$value div 72.27 * 25.4"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'bp'">
        <xsl:value-of select="$value div 72 * 25.4"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'dd'">
        <xsl:value-of select="$value * 1258 div 1157 * 25.4 div 72.27"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'pc'">
        <xsl:value-of select="12 * $value div 72.27 * 25.4"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'cc'">
        <xsl:value-of select="12 * $value * 1258 div 1157 * 25.4 div 72.27"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'linewidth'">
        <xsl:value-of select="$value * $linewidth"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'cropwidth'">
        <xsl:value-of select="$value * ($pagewidth + $cropgap)"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'textheight'">
        <xsl:value-of select="$value * $Top"/>
      </xsl:when>
      <xsl:when test="$elem/@unit eq 'cropheight'">
        <xsl:value-of select="$value * ($pageheight + 2 * $cropgap)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="0"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!-- 
The function rounds the height of the box so that it is an integer number of lines where the height
of the first line may differ. The height and width of the box are also returned as a decimal numbers
in milimeters.
  -->
  <xsl:function name="zw:roundheight" as="element()">
    <xsl:param name="elem" as="element()"/>
    <xsl:param name="splittopskip" as="xs:decimal"/>
    <xsl:variable name="boxlines" as="xs:decimal"
     select="round((zw:milimeters($elem/height) - $splittopskip) div $baselineskip)"/>
    <xsl:variable name="indent" as="xs:decimal"
     select="if ($elem/indent) then zw:milimeters($elem/indent) else 0"/>
    <vbox number="{$boxlines+1}" height="{$boxlines * $baselineskip + $splittopskip}"
      width="{$indent + zw:milimeters($elem/width)}">
      <xsl:copy-of select="$elem/*"/>
    </vbox>
  </xsl:function>
  
  <!--
This function gets the sequence of <text> elements and returns a single <vbox> with @height and @width
attributes as decimal numbers in milimeters. The element contains the <parshape> elements with
@number, @indent, and @width attributes. The @width and @indent attributes are strings in TeX
notation. If the sequence contains several <text> elements, they will all be merged into a single
<vbox>. Do not supply an empty sequence although the function can handle it and returns
<vbox height="0" width="0"/>
  -->
  <xsl:function name="zw:vbox" as="element()">
    <xsl:param name="text" as="element()*"/>
    <xsl:choose>
      <xsl:when test="count($text) eq 0">
        <vbox height="0" width="0"/>
      </xsl:when>
      <xsl:when test="count($text) eq 1">
        <xsl:variable name="vbox" as="element()*">
          <xsl:choose>
            <xsl:when test="$text/height">
              <xsl:sequence select="zw:roundheight($text, $topskip)"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:sequence select="zw:roundheight($text/shape[1], $topskip)"/>
              <xsl:sequence select="zw:roundheight($text/shape[2 to last()], $baselineskip)"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <vbox height="{sum($vbox/@height)}" width="{max($vbox/@width)}">
          <xsl:apply-templates select="$vbox" mode="vbox2parshape"/>
        </vbox>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="vboxes" as="element()*"
         select="for $t in $text return zw:vbox($t)"/>
        <vbox height="{sum($vboxes/@height)}" width="{max($vboxes/@vidth)}">
          <xsl:copy-of select="$vboxes/parshape"/>
        </vbox>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:template match="vbox" mode="vbox2parshape">
    <parshape number="{@number}" indent="{zw:TeXdim(indent)}" width="{zw:TeXdim(width)}"/>    
  </xsl:template>
  
  <!-- Compose value and TeX unit, return 0mm on empty sequence -->
  <xsl:function name="zw:TeXdim" as="xs:string">
    <xsl:param name="dim" as="element()?"/>
    <xsl:choose>
      <xsl:when test="empty($dim)">
        <xsl:text>0mm</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="value" as="xs:decimal" select="if ($dim/@value) then $dim/@value else 1"/>
        <xsl:variable name="unit-sep" as="xs:string"
         select="if (string-length($dim/@unit) &gt; 2) then '\' else ''"/>
        <xsl:value-of select="concat($value, $unit-sep, $dim/@unit)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <!-- Definition of a TeX dimension (number) -->
  <xsl:function name="zw:defTeXdim" as="xs:string">
    <xsl:param name="macro" as="xs:string"/>
    <xsl:param name="value" as="xs:decimal"/>
    <xsl:value-of select="concat('\def\', $macro, '{', $value, '}&#10;')"/>
  </xsl:function>
  
  <!-- Text nodes -->
  <xsl:template match="text()">
    <xsl:variable name="lang" as="node()*" select="(ancestor-or-self::*/@lang)[last()]"/>
    <xsl:choose>
      <xsl:when test="$lang = 'cs'">
        <xsl:value-of select="replace(replace(., '([ (,][AaIiKkOoSsUuVvZz])[ &#x0d;&#x0a;]+', '$1~'),
          '(~[AaIiKkOoSsUuVvZz])[ &#x0d;&#x0a;]+', '$1~')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!-- Ignore, just information -->
  <xsl:template match="images"/>
</xsl:stylesheet>
